package com.ruoyi.common.core.domain.entity;

import com.easy.mongodb.common.annotation.JoinCondition;
import com.easy.mongodb.common.annotation.Table;
import com.easy.mongodb.common.annotation.TableField;
import com.easy.mongodb.common.enums.FieldType;
import com.ruoyi.common.annotation.Excel;
import com.ruoyi.common.annotation.Excel.Type;
import com.ruoyi.common.annotation.Excels;
import com.ruoyi.common.core.domain.BaseEntity;
import com.ruoyi.common.xss.Xss;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

import javax.validation.constraints.Email;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 用户对象 sys_user
 *
 * @author ruoyi
 */
@Table(value = "sys_user")
public class SysUser extends BaseEntity {
    private static final long serialVersionUID = 1L;

    /** 部门ID */
    @Excel(name = "部门编号", type = Type.IMPORT)
    @TableField(fieldType = FieldType.OBJECT_ID)
    private String deptId;

    /** 用户账号 */
    @Excel(name = "登录名称")
    private String userName;

    /** 用户昵称 */
    @Excel(name = "用户名称")
    private String nickName;

    /** 用户邮箱 */
    @Excel(name = "用户邮箱")
    private String email;

    /** 手机号码 */
    @Excel(name = "手机号码")
    private String phonenumber;

    /** 用户性别 */
    @Excel(name = "用户性别", readConverterExp = "0=男,1=女,2=未知")
    private String sex;

    /** 用户头像 */
    private String avatar;

    /** 密码 */
    private String password;

    /** 帐号状态（0正常 1停用） */
    @Excel(name = "帐号状态", readConverterExp = "0=正常,1=停用")
    private String status;

    /** 删除标志（0代表存在 2代表删除） */
    private String delFlag;

    /** 最后登录IP */
    @Excel(name = "最后登录IP", type = Type.EXPORT)
    private String loginIp;

    /** 最后登录时间 */
    @Excel(name = "最后登录时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss", type = Type.EXPORT)
    @TableField
    private Date loginDate;

    /** 部门对象 */
    @Excels({
            @Excel(name = "部门名称", targetAttr = "deptName", type = Type.EXPORT),
            @Excel(name = "部门负责人", targetAttr = "leader", type = Type.EXPORT)
    })
    @TableField(
            fieldType = FieldType.JOIN,
            condition = @JoinCondition(
                    selfField = "dept_id", joinField = "_id"
            ))
    private SysDept dept;

    /** 角色对象 */
    @TableField(
            fieldType = FieldType.JOIN,
            condition = @JoinCondition(
                    entity = SysRole.class,
                    midEntity = SysUserRole.class, selfField = "_id", selfMidField = "user_id", joinMidField = "role_id", joinField = "_id"
            ))
    private List<SysRole> roles;

    /** 角色组 */
//    @TableField(
//            fieldType = FieldType.JOIN,
//            condition = @JoinCondition(
//                    entity = SysRole.class, field = "_id",
//                    midEntity = SysUserRole.class, selfField = "_id", selfMidField = "user_id", joinMidField = "role_id", joinField = "_id"
//            ))
    private String[] roleIds;

    /** 岗位组 */
    @TableField(
            fieldType = FieldType.JOIN,
            condition = @JoinCondition(
                    entity = SysPost.class,
                    midEntity = SysUserPost.class, selfField = "_id", selfMidField = "user_id", joinMidField = "post_id", joinField = "_id"
            ))
    private List<SysPost> postIds;

    /** 角色ID */
    private String roleId;
    private Boolean admin;

    public SysUser() {

    }

    public Boolean getAdmin() {
        return this.admin;
    }

    public void setAdmin(Boolean admin) {
        this.admin = admin;
    }

    public SysUser(String userId) {
        this.setId(userId);
    }

    public boolean isAdmin() {
        return Objects.isNull(this.admin) ? false : this.admin;
    }

    public String getDeptId() {
        return this.deptId;
    }

    public void setDeptId(String deptId) {
        this.deptId = deptId;
    }

    @Xss(message = "用户昵称不能包含脚本字符")
    @Size(min = 0, max = 30, message = "用户昵称长度不能超过30个字符")
    public String getNickName() {
        return this.nickName;
    }

    public void setNickName(String nickName) {
        this.nickName = nickName;
    }

    @Xss(message = "用户账号不能包含脚本字符")
    @NotBlank(message = "用户账号不能为空")
    @Size(min = 0, max = 30, message = "用户账号长度不能超过30个字符")
    public String getUserName() {
        return this.userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    @Email(message = "邮箱格式不正确")
    @Size(min = 0, max = 50, message = "邮箱长度不能超过50个字符")
    public String getEmail() {
        return this.email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    @Size(min = 0, max = 11, message = "手机号码长度不能超过11个字符")
    public String getPhonenumber() {
        return this.phonenumber;
    }

    public void setPhonenumber(String phonenumber) {
        this.phonenumber = phonenumber;
    }

    public String getSex() {
        return this.sex;
    }

    public void setSex(String sex) {
        this.sex = sex;
    }

    public String getAvatar() {
        return this.avatar;
    }

    public void setAvatar(String avatar) {
        this.avatar = avatar;
    }

    public String getPassword() {
        return this.password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getStatus() {
        return this.status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getDelFlag() {
        return this.delFlag;
    }

    public void setDelFlag(String delFlag) {
        this.delFlag = delFlag;
    }

    public String getLoginIp() {
        return this.loginIp;
    }

    public void setLoginIp(String loginIp) {
        this.loginIp = loginIp;
    }

    public Date getLoginDate() {
        return this.loginDate;
    }

    public void setLoginDate(Date loginDate) {
        this.loginDate = loginDate;
    }

    public SysDept getDept() {
        return this.dept;
    }

    public void setDept(SysDept dept) {
        this.dept = dept;
    }

    public List<SysRole> getRoles() {
        return this.roles;
    }

    public void setRoles(List<SysRole> roles) {
        this.roles = roles;
    }

    public String[] getRoleIds() {
        return this.roleIds;
    }

    public void setRoleIds(String[] roleIds) {
        this.roleIds = roleIds;
    }

    public List<SysPost> getPostIds() {
        return this.postIds;
    }

    public void setPostIds(List<SysPost> postIds) {
        this.postIds = postIds;
    }

    public String getRoleId() {
        return this.roleId;
    }

    public void setRoleId(String roleId) {
        this.roleId = roleId;
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this, ToStringStyle.MULTI_LINE_STYLE)
                .append("userId", this.getId())
                .append("deptId", this.getDeptId())
                .append("userName", this.getUserName())
                .append("nickName", this.getNickName())
                .append("email", this.getEmail())
                .append("phonenumber", this.getPhonenumber())
                .append("sex", this.getSex())
                .append("avatar", this.getAvatar())
                .append("password", this.getPassword())
                .append("status", this.getStatus())
                .append("delFlag", this.getDelFlag())
                .append("loginIp", this.getLoginIp())
                .append("loginDate", this.getLoginDate())
                .append("createBy", this.getCreateBy())
                .append("createTime", this.getCreateTime())
                .append("updateBy", this.getUpdateBy())
                .append("updateTime", this.getUpdateTime())
                .append("remark", this.getRemark())
                .append("dept", this.getDept())
                .toString();
    }
}
